@@ -2,7 +2,7 @@
  * bindings_java_signal.c
  *
  * Copyright (c) 1998-2005 The java-gnome Team
- * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -24,6 +24,7 @@
 #include <string.h>
 #include <glib.h>
 #include <glib-object.h>
+#include <gtk/gtk.h>
 #include "bindings_java.h"
 
 typedef struct  {
@@ -40,7 +41,7 @@ typedef struct  {
 	jchar		returnType;
  	
  	/*
- 	 * The Class upon which we invoke the [static] "handlerBlah" method.
+ 	 * The Class upon which we invoke the [static] "receiveBlah" method.
  	 */
  	jclass		receiver;
  
@@ -117,7 +118,7 @@ bindings_java_marshaller
 	 * Build the parameters for the callback. The signature of the
 	 * handlers on the Java side for a signal "name" would be:
 	 * 
-	 * 	handlerName(Signal handler, type arg0, type arg1, ...)
+	 * 	receiveName(Signal handler, type arg0, type arg1, ...)
 	 *
 	 * Note that arg0 is universally the source object (in otherwords, a
 	 * method function where the first argument is always a reference to
@@ -186,6 +187,7 @@ bindings_java_marshaller
       			break;
 
 		case G_TYPE_OBJECT:
+		case G_TYPE_INTERFACE:
 			/*
 			 * GObjects are just pointers, and so we pass up the
 			 * address across the boundary to be looked up and
@@ -207,7 +209,7 @@ bindings_java_marshaller
 			
 		default:
 			/*
-			 * Unrecogniz2ed. Probably means we need to add a clause above.
+			 * Unrecognized. Probably means we need to add a clause above.
 			 */
 			g_warning("Don't know how to marshal a %s", g_type_name(type));
 			jargs[i+1].l = 0;
@@ -230,7 +232,7 @@ bindings_java_marshaller
 
 	case 'Z':
 		/*
-		 * boolean return signal2s
+		 * boolean return signals
 		 */
 		_b = (*env)->CallStaticBooleanMethodA(env, bjc->receiver, bjc->method, jargs);		
 		if (_b == JNI_TRUE) {
@@ -290,19 +292,6 @@ bindings_java_marshaller
 		break;
 	}
 
-	/*
-	 * Now, check if an exception occurred in the callback and print the
-	 * stack trace in case it did. Otherwise, the stack trace gets swallowed
-	 * (ie, we're now back at GTK which is where the code path originated.
-	 * There's nowhere else for the signal to go, is there? Well, we could
-	 * call gtk_main_quit(), but that would probably be considered rude.
-	 */
-
-	if ((*env)->ExceptionOccurred(env)) {
-		(*env)->ExceptionDescribe(env);
-		(*env)->ExceptionClear(env);
-	}
-
 	/*
 	 * Cleanup
 	 */
@@ -320,8 +309,31 @@ bindings_java_marshaller
 	}
 	
 	/*
-	 * And don't need to free jargs - we alloca()'d it
+	 * Don't need to free jargs - we alloca()'d it
 	 */
+	
+	/*
+	 * Now, check if an exception occurred in the callback. There's a
+	 * catch: because we're in native code right now [care of the call to
+	 * gtk_main()] the exception gets swallowed until we return from that
+	 * native call. So we call the function which causes the main loop to
+	 * terminate, with the result that the exception will propegate out
+	 * and, yes, probably halt the program.
+	 * 
+	 * This is abrupt, but it is deliberate: we need to force developers
+	 * to deal with criticals emitted by the underlying libraries.
+	 * Otherwise, the next thing that is likely to happen is a
+	 * segmentation fault, and not only does that crash the "program" but
+	 * it takes out the Java Virtual Machine running it. People don't
+	 * like VM crashes.
+	 * 
+	 * Uncaught exceptions of any kind leaving a signal handler are to be
+	 * considered programmer error and will be fatal.
+	 */
+
+	if ((*env)->ExceptionOccurred(env)) {
+		gtk_main_quit();
+	}
 }
 
 /**
@@ -428,12 +440,12 @@ bindings_java_closure_new
 	
 
 	/*
-	 * the name of the methods we invoke is algorithmic: "handleName",
+	 * the name of the methods we invoke is algorithmic: "receiveName",
 	 * where Name is a PascalCase version of the signal name we were
 	 * passed in.
 	 */
 	 
-	buf = g_string_new("handle");
+	buf = g_string_new("receive");
 	
 	gchar** tokens = g_strsplit_set(name, "_-", -1);
 	
